#include <stdio.h>
#include <stdlib.h>
#include <alsa/asoundlib.h>

/* License: GPL-2 */

/* a large number of frames so we can batch more samples */
#define BUF_FRAMES_MAX 65536
/* sample rate used */
#define DEFAULT_RATE 22000
/* number of times the write can fail before the program exists */
#define ERR_TOLERANCE 2

main (int argc, char *argv[]) {
	int i;
	int err;
	char* buf;
	snd_pcm_t *playback_handle;
	snd_pcm_hw_params_t *hw_params;
	const char* device;
	int rate;
	snd_pcm_uframes_t buffer_frames;
	int no_writes;
	int err_last;

	if (argc > 3 || argc > 1 && (strcmp("-h", argv[1])==0 || strcmp("--help", argv[1])==0)) {
		printf("Usage: %s [device] [output-zeroes]\n", argv[0]);
		printf("  device       The audio device to use, defaults to \"default\".\n");
		printf("  output-zeros When set to \"1\" zeroed audio samples will be written to the\n");
		printf("               sound device. When set to \"0\" the device is opened and\n");
        printf("               prepared, but no samples are written. The default is 0 and is\n");
		printf("               apparently sufficient.\n");
		exit(1);
	}
	
	/* set the default params */
	no_writes = 1;
	device = "default";
	
	if (argc > 1) {
		device = argv[1];
	}

	if (argc > 2) {
		no_writes = atoi(argv[2]);
	}

	if ((err = snd_pcm_open (&playback_handle, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
		fprintf (stderr, "cannot open audio device %s (%s)\n", 
				 argv[1],
				 snd_strerror (err));
		exit (1);
	}

	printf("Using device \"%s\"\n", device);

	
	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
		fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	
	if ((err = snd_pcm_hw_params_any (playback_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	
	if ((err = snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
		fprintf (stderr, "cannot set access type (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	
	if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S8)) < 0) {
		fprintf (stderr, "cannot set sample format (%s)\n",
				 snd_strerror (err));
		exit (1);
	}

	rate = DEFAULT_RATE;
	if ((err = snd_pcm_hw_params_set_rate_near (playback_handle, hw_params, &rate, 0)) < 0) {
		fprintf (stderr, "cannot set sample rate (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	printf("rate selected: %d\n", rate);

	/* select mono */
	if ((err = snd_pcm_hw_params_set_channels (playback_handle, hw_params, 1)) < 0) {
		fprintf (stderr, "cannot set channel count (%s)\n",
				 snd_strerror (err));
		exit (1);
	}

	/* get the largest buffer we can */
	buffer_frames = BUF_FRAMES_MAX;
	if ((err = snd_pcm_hw_params_set_buffer_size_near(playback_handle, hw_params, &buffer_frames)) < 0) {
		fprintf (stderr, "cannot set buffer size (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	printf("buffer size: %d frames\n", (int)buffer_frames);

	/* set the device parameters */
	if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot set parameters (%s)\n",
				 snd_strerror (err));
		exit (1);
	}
	
	snd_pcm_hw_params_free (hw_params);
	
	if ((err = snd_pcm_prepare (playback_handle)) < 0) {
		fprintf (stderr, "cannot prepare audio interface for use (%s)\n",
				 snd_strerror (err));
		exit (1);
	}

	if (no_writes) {
		printf("Device will only be prepared, and not written.\n");
	} else {
		printf("Device will be sent zeroed audio.\n");
		/* in our case, every byte is a full buffer frame (mono 8bit) */
		buf = (char*)malloc(buffer_frames);
		memset(buf, 0, buffer_frames);
	}

	err_last = 0;
	while (1) {
		if (no_writes) {
			/* sleep an hour */
			sleep(60*60);
		} else {
			if ((err = snd_pcm_writei(playback_handle, buf, buffer_frames/2)) != buffer_frames/2) {
				if (err_last > ERR_TOLERANCE) {
					fprintf (stderr, "write to audio interface failed (%s)\n",
							 snd_strerror (err));
					exit (1);
				} else {
					err_last = 1;
				}
			} else {
				err_last = 0;
			}
		}
	}

	//we never actually make it here, but if we did this would be useful

	if (!no_writes) {
		free(buf);
	}
	
	snd_pcm_close (playback_handle);
	exit (0);
}
